local  Editor = Game:addState("Editor")

function Editor:enterState()
	self.top = 1
	self.bottom = GENERATE_BOX_BOTTOM - GENERATE_BOX_TOP
	self.left = 1
	self.right = GameWidth
	
	self.towerType = TOWER_POWER
	self.Towers = {}
end

function Editor:update()
end

function Editor:draw()
    love.graphics.draw(Graphics.backgrnd,0,0,0,1,1)

	love.graphics.setColor(Color.menu_fore_color)
	love.graphics.setLine(3, "rough")
	love.graphics.rectangle("line", self.left, self.top, self.right - self.left, self.bottom - self.top)

	love.graphics.setColor(Color.menu_back_color)

	love.graphics.setFont(Font.state_hint_font)
	love.graphics.printf(
						"1: Power (no more than 1)\n" ..
						"2: Verticle Weapon Tower\n" ..
						"3: Rotate Weapon Tower\n" ..
						"4: Missile Weapon Tower\n" ..
						"5: Shield Tower\n" ..
						"6: Brick (should no other towers)\n" ..
						"s: print on console\n" ..
						"backspace: delete last one\n" ..
						"ESC: back to menu", 30, 350, GameWidth - 30)

	love.graphics.setColor(Color.default)

	self:drawObjs(self.Towers)
	
	love.graphics.setColor(Color.default)
end

function Editor:drawObjs(objs)
	for i, obj in ipairs(objs) do
		obj:draw()
	end
end

function Editor:Print()
	print("\t{")
	
	for i, obj in ipairs(self.Towers) do
		print("\t\t{" .. obj.towerType .. ", " .. obj._x .. ", " .. (obj._y - self.top + GENERATE_BOX_TOP) .. "},")
	end
	
	print("\t},\n")
end

function Editor:keypressed(key)
	if key == "s" then
		-- save, just print it on console
		self:Print()
	elseif key == "1" then
		-- tower
		self.towerType = TOWER_POWER
	elseif key == "2" then
		-- verticle
		self.towerType = TOWER_VERTICAL
	elseif key == "3" then
		-- rotate
		self.towerType = TOWER_ROTATE
	elseif key == "4" then
		-- missile
		self.towerType = TOWER_MISSILE
	elseif key == "5" then
		-- shield
		self.towerType = TOWER_SHIELD
	elseif key == "6" then
		-- brick
		self.towerType = OBJTYPE_BRICK
	elseif key == "backspace" then
		-- delete last one
		if #self.Towers > 0 then
			table.remove(self.Towers)
		end
	elseif key == "escape" then
		Game:popState("PauseScreen")
	end
end

function Editor:keyreleased(key)
end

function Editor:mousepressed(x,y,button )
	if x > self.left and x < self.right and	y > self.top and y < self.bottom then
		if self.towerType == TOWER_POWER then
			local tower = Defender:new()
			tower:setPos(x, y)
			table.insert(self.Towers, tower)
		elseif self.towerType == TOWER_VERTICAL then
			local tower = Tower:new(TOWER_VERTICAL, CT_WHITE)
			tower:setPos(x, y)
			table.insert(self.Towers, tower)
		elseif self.towerType == TOWER_ROTATE then
			local tower = Tower:new(TOWER_ROTATE, CT_WHITE)
			tower:setPos(x, y)
			table.insert(self.Towers, tower)
		elseif self.towerType == TOWER_MISSILE then
			local tower = Tower:new(TOWER_MISSILE, CT_WHITE)
			tower:setPos(x, y)
			table.insert(self.Towers, tower)
		elseif self.towerType == TOWER_SHIELD then
			local tower = Tower:new(TOWER_SHIELD, CT_WHITE)
			tower:setPos(x, y)
			table.insert(self.Towers, tower)
		elseif self.towerType == OBJTYPE_BRICK then
			local brick = Brick:new(TOWER_MISSILE, CT_WHITE)
			brick:setPos(x, y)
			table.insert(self.Towers, brick)
		else
			error("Unknown self.towerType: ", self.towerType)
		end
	end
end

function Editor:exitState()
end


