ExplodeParticle = class("ExplodeParticle")

Particles = {}

function ExplodeParticle:initialize(image, x, y, buffer)
	self.x = x
	self.y = y
	
	self.particleSystem = love.graphics.newParticleSystem(image, buffer)
	self.particleSystem:setPosition(x, y)

end

function ExplodeParticle:shouldBeDelete()
	return not self.particleSystem:isActive()
end

function ExplodeParticle:update(timeStep)
    self.particleSystem:update(timeStep)

	if self.tickCount ~= nil then
		self.tickCount = self.tickCount - 1
		if self.tickCount == 0 then
			self.particleSystem:setEmissionRate(0)
		end
	end
end

function ExplodeParticle:draw()
    love.graphics.draw(self.particleSystem, 0, 0)
end

function AddBulletExplode(x, y)
	PlayBulletExplodeSfx()

	local particle = ExplodeParticle:new(Graphics.bullet_explode, x, y, 10)
	
	particle.particleSystem:setLifetime(0.2)

    particle.particleSystem:setEmissionRate(100)
    particle.particleSystem:setSpeed(30, 60)
    particle.particleSystem:setSize(1, 1)

    particle.particleSystem:setParticleLife(0.3, 0.6)
    particle.particleSystem:setSpread(math.pi * 2)
    particle.particleSystem:start()
	
	table.insert(Particles, particle)
end

function AddTowerExplode(x, y)
	PlayExplodeSfx()
	
	local particle = ExplodeParticle:new(Graphics.tower_explode, x, y, 20)
	
	particle.particleSystem:setLifetime(1)

    particle.particleSystem:setEmissionRate(50)
    particle.particleSystem:setSpeed(15, 20)
    particle.particleSystem:setSize(0.7, 1.3, 1)

    particle.particleSystem:setParticleLife(1.0, 2)
    particle.particleSystem:setSpread(math.pi * 2)

	particle.particleSystem:setColor( 0, 192, 0, 128, 0, 192, 255, 0 )
	
    particle.particleSystem:start()
	
	table.insert(Particles, particle)
end

function AddWinExplode(x, y)
	PlayExplodeSfx()

	local particle = ExplodeParticle:new(Graphics.tower_explode, x, y, 200)
	
	particle.tickCount = 30
	
	particle.particleSystem:setLifetime(4)

    particle.particleSystem:setEmissionRate(20)
    particle.particleSystem:setSpeed(15, 20)
    particle.particleSystem:setSize(0.7, 1.3, 1)

    particle.particleSystem:setParticleLife(2.0, 3)
    particle.particleSystem:setSpread(math.pi * 2)

	particle.particleSystem:setColor( math.random(255), math.random(255), math.random(255), 192,
		math.random(255), math.random(255), math.random(255), 0 )
	
    particle.particleSystem:start()
	
	table.insert(Particles, particle)
end

function UpdateParticles()
	for i=#Particles, 1, -1 do
		local p = Particles[i]
		if p:shouldBeDelete() then
			table.remove(Particles, i)
		else
			p:update(0.05)
		end
	end
end

function DrawParticles()
	for i, p in ipairs(Particles) do
		p:draw()
	end
end

function DeleteAllParticles()
	for i=#Particles, 1, -1 do
		table.remove(Particles, i)
	end
end

