local Play = Game:addState("Play")

function Play:enterState()
	self:initStartState()
end

function Play:initStartState()
	OrangeBullets = {}
	BlueBullets = {}
	Player = PlayerClass:new()
	Players = {Player}
	WhiteTowers = {}
	Defenders = {}
	Bricks = {}
	
	-- tmp global array? -- need refactor?
	AllPlayers = {}
	AllDefenders = {}

	-- player
	Player._y = GameHeight / 4 * 3

	-- init towers
	local tower1 = Tower:new(TOWER_VERTICAL, CT_WHITE)
	tower1:setX(GameWidth / 3)
	table.insert(WhiteTowers, tower1)
	local tower2 = Tower:new(TOWER_VERTICAL, CT_WHITE)
	tower2:setX(GameWidth / 3 * 2)
	table.insert(WhiteTowers, tower2)
	
	-- counting
	self.deletedDefenders = 0
	
	self.generateBrick = false
end

function Play:update()
	self:generateNewTowers()
	
	-- clear tmp array
	AllPlayers = {}
	AllDefenders = {}
	self:collectAll(Players, AllPlayers)
	self:collectAll(Defenders, AllDefenders)

	self:updateObjs(Bricks)
	Player:moveByInput()
	self:updateObjs(Defenders)
	Player:update()
	self:attractWhiteTowers(Defenders, WhiteTowers)
	self:updateWhiteTowers(WhiteTowers)
	self:updateBullets(OrangeBullets, AllDefenders)
	self:updateBullets(BlueBullets, AllPlayers)
	self:collideBetweenTowersGroups(AllPlayers, AllDefenders)
	self:collideBetweenTowersGroups(AllPlayers, Bricks)
	self:collideBetweenTowersGroups(AllDefenders, Bricks)
	
	self:updateTarget(Players, Defenders)
	self:updateTarget(Defenders, Players)
	
	self:removeDeadTowers(Players)
	local deletedTowers = self:removeDeadTowers(Defenders)
	self.deletedDefenders = self.deletedDefenders + deletedTowers
	
	self:deleteShouldBeDelete(WhiteTowers)
	self:deleteShouldBeDelete(Bricks)
	
	-- clear tmp array
	AllPlayers = {}
	AllDefenders = {}
	
	UpdateParticles()
	
	self:checkResult()
end

function Play:generateNewTowers()
	if self.generateBrick == true then
	 	if #Defenders == 0 then
			self:generateBricks()
			self.generateBrick = not self.generateBrick
		end
	else
		if #Bricks == 0 then
			self:generateDefender()
			self.generateBrick = not self.generateBrick
		end
	end
	
	if math.random() < 0.001 then
		tmpr = math.random()
		if tmpr < 0.4 then
			local tower1 = Tower:new(TOWER_VERTICAL, CT_WHITE)
			local x = math.random(10, GameWidth - 10)
			tower1:setPos(math.random(10, GameWidth - 10), 0)
			table.insert(WhiteTowers, tower1)
		elseif tmpr < 0.8 then
			local tower1 = Tower:new(TOWER_ROTATE, CT_WHITE)
			tower1:setPos(math.random(10, GameWidth - 10), 0)
			table.insert(WhiteTowers, tower1)
		elseif tmpr < 0.9 then
			local tower1 = Tower:new(TOWER_MISSILE, CT_WHITE)
			tower1:setPos(math.random(10, GameWidth - 10), 0)
			table.insert(WhiteTowers, tower1)
		else
			local tower1 = Tower:new(TOWER_SHIELD, CT_WHITE)
			tower1:setPos(math.random(10, GameWidth - 10), 0)
			table.insert(WhiteTowers, tower1)
		end
	end
end

GENERATE_BOX_TOP = -250
GENERATE_BOX_BOTTOM = -30

function Play:generateBricks()
	local index = self.deletedDefenders
	local brickDefine = BrickDefines[index]
	if brickDefine ~= nil then
		-- generate by predefine
		for i, obj in ipairs(brickDefine) do
			local towerType, x, y = unpack(obj)
			if towerType == OBJTYPE_BRICK then
				local brick = Brick:new()
				brick:setPos(x, y)
				table.insert(Bricks, brick)
			else
				error("Other tower in bricks' define!")
			end
		end
	else
		-- generate by program
		for i=1, self.deletedDefenders * 2 do
			local brick = Brick:new()
			brick:setPos(math.random(10, GameWidth - 10), math.random(GENERATE_BOX_TOP, GENERATE_BOX_BOTTOM))
			table.insert(Bricks, brick)
		end
	end
end

function Play:generateDefender()
	local index = self.deletedDefenders + 1
	local defenderDefine = DefenderDefines[index]
	if defenderDefine ~= nil then
		-- generate by predefine
		local defendersCnt = 0
		for i, obj in ipairs(defenderDefine) do
			local towerType, x, y = unpack(obj)
			if towerType == TOWER_POWER then
				local defender = Defender:new()
				defender:setPos(x, y)
				table.insert(Defenders, defender)
				defendersCnt = defendersCnt + 1
			else
				local tower = Tower:new(towerType, CT_WHITE)
				tower:setPos(x, y)
				table.insert(WhiteTowers, tower)
			end
		end
		if defendersCnt == 0 then
			error("No defenders defined!", index)
		elseif defendersCnt > 1 then
			error("More than one defender defined!", index)
		end
	else
		-- generate by program
		if #Defenders == 0 then
			local defender = Defender:new()
			defender:setPos(math.random(10, GameWidth - 10), GENERATE_BOX_TOP)
			table.insert(Defenders, defender)
		end

		for i=1, (self.deletedDefenders + 1) * 2 do
			tmpr = math.random()
			if tmpr < 0.4 then
				local tower1 = Tower:new(TOWER_VERTICAL, CT_WHITE)
				local x = math.random(10, GameWidth - 10)
				tower1:setPos(math.random(10, GameWidth - 10), math.random(GENERATE_BOX_TOP, GENERATE_BOX_BOTTOM))
				table.insert(WhiteTowers, tower1)
			elseif tmpr < 0.8 then
				local tower1 = Tower:new(TOWER_ROTATE, CT_WHITE)
				tower1:setPos(math.random(10, GameWidth - 10), math.random(GENERATE_BOX_TOP, GENERATE_BOX_BOTTOM))
				table.insert(WhiteTowers, tower1)
			elseif tmpr < 0.9 then
				local tower1 = Tower:new(TOWER_MISSILE, CT_WHITE)
				tower1:setPos(math.random(10, GameWidth - 10), math.random(GENERATE_BOX_TOP, GENERATE_BOX_BOTTOM))
				table.insert(WhiteTowers, tower1)
			else
				local tower1 = Tower:new(TOWER_SHIELD, CT_WHITE)
				tower1:setPos(math.random(10, GameWidth - 10), math.random(GENERATE_BOX_TOP, GENERATE_BOX_BOTTOM))
				table.insert(WhiteTowers, tower1)
			end
		end
	end
end

DEFENDERS_DESTROYED_TO_SHOW_STORYBOARD1 = 2
DEFENDERS_DESTROYED_TO_WIN = 10
--DEFENDERS_DESTROYED_TO_SHOW_STODYBOARD1 = 1
--DEFENDERS_DESTROYED_TO_WIN = 3
StoryBoard1Displayed = false

function Play:checkResult()
	if #Particles == 0 then
		if #Players == 0 then
			Game:pushState("FailScreen")
		elseif self.deletedDefenders >= DEFENDERS_DESTROYED_TO_SHOW_STORYBOARD1 and StoryBoard1Displayed == false then
			Game:pushState("StoryBoard1")
			StoryBoard1Displayed = true
		elseif self.deletedDefenders >= DEFENDERS_DESTROYED_TO_WIN then
			Game:pushState("FinishScreen")
		end
	end
end

function Play:collideBetweenTowersGroups(towers1, towers2)
	for i, t1 in ipairs(towers1) do
		for j, t2 in ipairs(towers2) do
			if t1.dead == false and t2.dead == false and t1:collideWith(t2) then
				local t1power = t1.power
				local t2power = t2.power
				t1:reducePower(t2power)
				t2:reducePower(t1power)
				
				local explodeHappened = false
				if t1.dead == true then
					explodeHappened = true
					AddTowerExplode(t1._x, t1._y)
				end
				
				if t2.dead == true then
					explodeHappened = true
					AddTowerExplode(t2._x, t2._y)
				end
				
				if explodeHappened == true then
					if towers1 == Bricks or towers2 == Bricks then
						PlayBrickSfx()
					else
						PlayTowerSfx()
					end
				end
			end
		end
	end
end

function Play:removeDeadTowers(towers)
	local deletedTowers = 0
	for i=#towers, 1,-1 do
		local tower = towers[i]
		if tower.dead == true then
			tower:addFollowersToWhiteTowers()
			table.remove(towers, i)
			deletedTowers = deletedTowers + 1
		else
			tower:removeDeadFollowers()
		end
	end
	
	return deletedTowers
end

function Play:updateTarget(srcPowers, tgtPowers)
	for i, srcPower in ipairs(srcPowers) do
		local minDist = 1000000
		local minTarget = nil
		for j, tgtPower in ipairs(tgtPowers) do
			local dist = distanceFrom(srcPower._x, srcPower._y, tgtPower._x, tgtPower._y)
			if dist < minDist then
				minDist = dist
				minTarget = tgtPower
			end
		end
		
		srcPower:updateTarget(minTarget)
	end
end

function Play:collectAll(treeArray, objArray)
	for i, obj in ipairs(treeArray) do
		obj:collectAll(objArray)
		table.insert(objArray, obj)
	end	
end

function Play:updateObjs(objs)
	for i, obj in ipairs(objs) do
		obj:update()
	end
end

function Play:attractWhiteTowers(powerSources, wts)
	for i, powerSource in ipairs(powerSources) do
		for j, wt in ipairs(wts) do
			powerSource:tryAttractTower(wt)
		end
	end
end

function Play:deleteShouldBeDelete(objs)
	for i=#objs, 1,-1 do
		local obj = objs[i]
		if obj:shouldBeDelete() then
			table.remove(objs, i)
		end
	end
end

function Play:updateWhiteTowers(wts)
	for i=#wts, 1,-1 do
		local wt = wts[i]
		wt:update()
		if wt:shouldBeDelete() then
			table.remove(wts, i)
		elseif wt:tryToFollowTowers(AllPlayers) then
			if wt._y > 0 then
				PlayWhiteTowerSfx()
			end
			table.remove(wts, i)
		elseif wt:tryToFollowTowers(AllDefenders) then
			if wt._y > 0 then
				PlayWhiteTowerSfx()
			end
			table.remove(wts, i)
		end
	end
end

function Play:updateBullets(bullets, targetTowers)
	for i=#bullets, 1, -1 do
		local bullet = bullets[i]
		bullet:update()
		if bullet:shouldBeDelete() then
			table.remove(bullets, i)
		elseif bullet:testHit(targetTowers) then
			table.remove(bullets, i)
			AddBulletExplode(bullet._x, bullet._y)
		end
	end
end

function Play:draw()
	love.graphics.draw(Graphics.backgrnd,0,0,0,1,1)

	self:drawObjs(Bricks)
	
	love.graphics.setColor(Color.blue)
	self:drawObjs(Defenders)

	love.graphics.setColor(Color.orange)
	Player:draw()
	
	love.graphics.setColor(Color.white)
	self:drawObjs(WhiteTowers)
	
	love.graphics.setColor(Color.orange_bullet)
	self:drawObjs(OrangeBullets)
	
	love.graphics.setColor(Color.blue_bullet)
	self:drawObjs(BlueBullets)
	
	DrawParticles()
	
	love.graphics.setColor(Color.default)
end

function Play:drawObjs(objs)
	for i, obj in ipairs(objs) do
		obj:draw()
	end
end

function Play:keypressed(key)
	if key == "escape" then
		Game:pushState("PauseScreen")
	end
end

function Play:keyreleased(key)
end

function Play:exitState()
end


