Tower = class("Tower", Obj)

TOWER_VERTICAL = 1
TOWER_ROTATE = 2
TOWER_MISSILE = 3
TOWER_SHIELD = 4
TOWER_POWER = 5

TowerGraphPrefix = {
	"white_tower_", --TOWER_VERTICAL
	"white_tower_", --TOWER_ROTATE
	"white_tower_", --TOWER_MISSLE
	"white_tower_", --TOWER_SHIELD
	"white_power_", --TOWER_POWER
}

TowerR = {
	9, --TOWER_VERTICAL
	9, --TOWER_ROTATE
	9, --TOWER_MISSLE
	9, --TOWER_SHIELD
	12, --TOWER_POWER
}

CT_WHITE = 100
CT_ORANGE = 101
CT_BLUE = 102

function Tower:initialize(towerType, colorType)
    if towerType == TOWER_VERTICAL then
		graphName = TowerGraphPrefix[towerType] .. "1"
		Obj.initialize(self, Graphics[graphName], TowerR[towerType])
		self.weapon = Weapon:new(self, WT_VERTICAL)
	elseif towerType == TOWER_ROTATE then
		graphName = TowerGraphPrefix[towerType] .. "1"
		Obj.initialize(self, Graphics[graphName], TowerR[towerType])
		self.weapon = Weapon:new(self, WT_ROTATE)
	elseif towerType == TOWER_MISSILE then
		graphName = TowerGraphPrefix[towerType] .. "1"
		Obj.initialize(self, Graphics[graphName], TowerR[towerType])
		self.weapon = Weapon:new(self, WT_MISSILE)
	elseif towerType == TOWER_SHIELD then
		graphName = TowerGraphPrefix[towerType] .. "1"
		Obj.initialize(self, Graphics[graphName], TowerR[towerType])
		self.weapon = nil
		self.power = 150
		self.originalPower = 150
	elseif towerType == TOWER_POWER then
		graphName = TowerGraphPrefix[towerType] .. "1"
		Obj.initialize(self, Graphics[graphName], TowerR[towerType])
		self.weapon = nil
		if colorType == CT_WHITE then
			error("Should be no power of CT_WHITE!")
		end
		self.powerSource = self
	else
		error("Invalid towerType: " .. towerType)
	end
	
	self.towerType = towerType
	self.colorType = colorType
	
	self.dx = 0
	self.dy = 0
end

WHITE_TOWER_SPEED = .5

function Tower:update()
	if self.colorType == CT_WHITE then
		if self._y < GENERATE_BOX_BOTTOM then
			self:move(0 + self.dx, self.dy)
		else
			self:move(0 + self.dx, WHITE_TOWER_SPEED + self.dy)
		end
	end
	
	Obj.update(self)
	if self.weapon then
		self.weapon:update()
	end
end

function Tower:draw()
	Obj.draw(self)
	if self.weapon then
		self.weapon:draw()
	end
end

LIFE_4_IMAGE_1 = 0.6
LIFE_4_IMAGE_2 = 0.2

function Tower:_updateImageByPower()
	if self.power > self.originalPower * LIFE_4_IMAGE_1 then
		graphName = TowerGraphPrefix[self.towerType] .. "1"
		self.image = Graphics[graphName]
	elseif self.power > self.originalPower * LIFE_4_IMAGE_2 then
		graphName = TowerGraphPrefix[self.towerType] .. "2"
		self.image = Graphics[graphName]
	else
		graphName = TowerGraphPrefix[self.towerType] .. "3"
		self.image = Graphics[graphName]
	end
end

function Tower:follow(otherTower)
	if self.colorType ~= CT_WHITE then
		error("Only CT_WHITE could follow other tower!")
	end
	
	if otherTower.colorType == CT_WHITE then
		error("CT_WHITE could not be followed!")
	end
	
	if otherTower.powerSource == nil then
		error("The target tower have no power source!")
	end
	
	self.colorType = otherTower.colorType
	self.powerSource = otherTower.powerSource
	otherTower:addFollower(self)
end

function Tower:tryToFollowTowers(towers)
	for i, tower in ipairs(towers) do
		if self:collideWith(tower) then
			self:follow(tower)
			return true
		end
	end
	
	return false
end

function Tower:tryAttractTower(tower)
	error("Not available!")
end

function Tower:removeDeadFollowers()
	for i=#self.followers, 1,-1 do
		local follower = self.followers[i]
		if follower.dead == true then
			follower:addFollowersToWhiteTowers()
			table.remove(self.followers, i)
		else
			follower:removeDeadFollowers()
		end
	end
end

function Tower:addFollowersToWhiteTowers()
	for i=#self.followers, 1,-1 do
		local follower = self.followers[i]
		follower.colorType = CT_WHITE
		follower.powerSource = nil
		if follower.dead == false then
			follower.dx = (math.random() - 0.5) * WHITE_TOWER_SPEED
			follower.dy = (math.random() - 0.5) * WHITE_TOWER_SPEED
			table.insert(WhiteTowers, follower)
		end
		follower:addFollowersToWhiteTowers()
		table.remove(self.followers, i)
	end
end

