Weapon = class("Weapon", Obj)

WT_VERTICAL = 1
WT_ROTATE = 2
WT_MISSILE = 3

WeaponGraphName = {
	"white_weapon_vertical", --WT_VERTICAL
	"white_weapon_rotate", --WT_ROTATE
	"white_weapon_missle", --WT_MISSILE
}

function Weapon:initialize(tower, weaponType)	
    if weaponType == WT_VERTICAL then
		Obj.initialize(self, Graphics[WeaponGraphName[weaponType]], 0)
	elseif weaponType == WT_ROTATE then
		Obj.initialize(self, Graphics[WeaponGraphName[weaponType]], 0)
	elseif weaponType == WT_MISSILE then
		Obj.initialize(self, Graphics[WeaponGraphName[weaponType]], 0)
	else
		error("Invalid weaponType: " .. weaponType)
	end
	
	self.tower = tower
	self.weaponType = weaponType
	self.angle = -math.pi / 2
end

function Weapon:update()
	if self.tower.colorType ~= CT_WHITE then
		local thePower = self.tower.powerSource
		if thePower ~= nil then
			if self:updateTarget() then
			    if self.weaponType == WT_VERTICAL then
					if thePower.tickCount % 120 == 0 then
						Bullet:new(BT_LINE, self.tower.colorType,
							self.tower._x + 11 * math.cos(self.angle), self.tower._y + 11 * math.sin(self.angle),
							self.angle)
					end
				elseif self.weaponType == WT_ROTATE then
					if thePower.tickCount % 150 == 0 then
						Bullet:new(BT_LINE, self.tower.colorType,
							self.tower._x + 11 * math.cos(self.angle), self.tower._y + 11 * math.sin(self.angle),
							self.angle)
					end
				elseif self.weaponType == WT_MISSILE then
					if thePower.tickCount % 200 == 0 then
						Bullet:new(BT_MISSILE, self.tower.colorType,
							self.tower._x + 9 * math.cos(self.angle), self.tower._y + 9 * math.sin(self.angle),
							self.angle, thePower.targetPower)
					end
				else
					error("Invalid weaponType: " .. self.weaponType)
				end
			end
		else
			error("PowerSource is nil!")
		end
	end
end

function Weapon:draw()
	if self.tower.dead == false then
		love.graphics.draw(
			self.image,
			self.tower._x, self.tower._y,
			self.angle,
			1, 1,
			self.ox, self.oy)
	end
end

function Weapon:updateTarget()
	local thePower = self.tower.powerSource
	local targetPower = thePower.targetPower
	if targetPower == nil or targetPower.dead == true then
		return false
	else
	    if self.weaponType == WT_VERTICAL or self.weaponType == WT_MISSILE then
			if targetPower._y < self.tower._y then
				self.angle = -math.pi / 2
			else
				self.angle = math.pi / 2
			end
		elseif self.weaponType == WT_ROTATE then
			self.angle = math.atan2(targetPower._y - self.tower._y, targetPower._x - self.tower._x)
		else
			error("Invalid weaponType: " .. self.weaponType)
		end
		return true
	end
end

